--[[
BigBrother
Concept and original mod: Cryect
Currently maintained by: Treibh (Vek'nilash US)
Additional thanks:
    * All of the translators
    * Other wowace developers for assistance and bug fixes
    * Ahti and the other members of Cohors Praetoria (Vek'nilash US) for beta testing new versions of the mod
    * Thanks to vhaarr for helping Cryect out with reducing the length of code
    * Thanks to pastamancer for fixing the issues with Supreme Power Flasks and pointing in right direction for others
    * Window Resizing code based off the dragbar from violation
    * And also thanks to all those in #wowace for the various suggestions
]]

BigBrother = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0","FuBarPlugin-2.0")

local addon = BigBrother

local AceEvent = AceLibrary("AceEvent-2.0")
local RL = AceLibrary("Roster-2.1")
local aura = AceLibrary("SpecialEvents-Aura-2.0")
local parser = ParserLib:GetInstance("1.1")
local L = AceLibrary("AceLocale-2.2"):new("BigBrother")
local BS = LibStub("LibBabble-Spell-3.0"):GetLookupTable()

-- table of flasks that are checked
local buffs = {
	L["Mr Pinchy's Blessing"],
	L["Chromatic Resistance"],
	L["Distilled Wisdom"],
	L["Flask of Fortification"],
	L["Flask of Mighty Restoration"],
	L["Flask of Relentless Assault"],
	L["Supreme Power"],
	L["Flask of the Titans"],
	L["Supreme Power of Shattrath"],
	L["Fortification of Shattrath"],
	L["Mighty Restoration of Shattrath"],
	L["Relentless Assault of Shattrath"],
	-- new for 2.2
	L["Flask of Pure Death"],
	L["Flask of Blinding Light"],
	L["Chromatic Wonder"],
  -- Gruul
  L["Unstable Flask of the Bandit"],
  L["Unstable Flask of the Beast"],
  L["Unstable Flask of the Elder"],
  L["Unstable Flask of the Physician"],
  L["Unstable Flask of the Soldier"],
  L["Unstable Flask of the Sorcerer"],
}

-- table of elixirs that are checked
local elixirs = {
  L["Fel Strength Elixir"],
  L["Onslaught Elixir"],
  L["Major Strength"],
  L["Elixir of Major Strength"],
  L["Major Agility"],
  L["Elixir of Major Agility"],
  L["Elixir of Mastery"],
  L["Major Firepower"],
  L["Elixir of Major Firepower"],
  L["Major Shadow Power"],
  L["Elixir of Major Shadow Power"],
  L["Major Frost Power"],
  L["Elixir of Major Frost Power"],
  L["Adept's Elixir"],
  L["Arcane Elixir"],
  L["Greater Arcane Elixir"],
  L["Healing Power"],
  L["Elixir of Healing Power"],
  L["Elixir of the Mongoose"],
  L["Major Armor"],
  L["Elixir of Major Defense"],
  L["Greater Armor"],
  L["Elixir of Superior Defense"],
  L["Greater Mana Regeneration"],
  L["Elixir of Major Mageblood"],
  L["Mana Regeneration"],
  L["Mageblood Potion"],
  L["Earthen Elixir"],
  L["Elixir of Draenic Wisdom"],
  L["Elixir of Major Fortitude"],
  L["Greater Intellect"],
  L["Elixir of Empowerment"],
  L["Empowerment"],
  L["Elixir of Ironskin"],
  L["Elixir of Greater Firepower"],
}

local color = "|cffff8040%s|r"

-- FuBar stuff
addon.name = "BigBrother"
addon.hasIcon = true
addon.hasNoColor = true
addon.clickableTooltip = false
addon.independentProfile = true
addon.cannotDetachTooltip = true
addon.hideWithoutStandby = true

function addon:OnClick(button)
	ToggleBuffWindow()
end

function addon:OnTextUpdate()
	self:SetText("BigBrother")
end

-- AceDB stuff
addon:RegisterDB("BigBrotherDB")
addon:RegisterDefaults("profile", {
  PolyBreak = true,
  Misdirect = true,
  Groups = {true, true, true, true, true, true, true, true},
  PolyOut = {true, false, false, false, false},
  CheckFlasks = true,
  CheckElixirs = true,
  CheckFood = true,
})

-- ACE options menu
local options = {
  type = 'group',
  handler = BigBrother,
  args = {
    flaskcheck = {
      name = L["Flask Check"],
      desc = L["Checks for flasks, elixirs and food buffs."],
      type = 'group',
      args = {
        self = {
          name = L["Self"],
          desc = L["Reports result only to yourself."],
          type = 'execute',
          func = "FlaskCheck",
          passValue = "SELF",
        },
        party = {
          name = L["Party"],
          desc = L["Reports result to your party."],
          type = 'execute',
          func = "FlaskCheck",
          disabled = function() return GetNumPartyMembers()==0 end,
          passValue = "PARTY",
        },
        raid = {
          name = L["Raid"],
          desc = L["Reports result to your raid."],
          type = 'execute',
          func = "FlaskCheck",
          disabled = function() return GetNumRaidMembers()==0 end,
          passValue = "RAID",
        },
        guild = {
          name = L["Guild"],
          desc = L["Reports result to guild chat."],
          type = 'execute',
          func = "FlaskCheck",
          passValue = "GUILD",
        },
        officer = {
          name = L["Officer"],
          desc = L["Reports result to officer chat."],
          type = 'execute',
          func = "FlaskCheck",
          passValue = "OFFICER",
        },
        whisper = {
          name = L["Whisper"],
          desc = L["Reports result to the currently targeted individual."],
          type = 'execute',
          func = "FlaskCheck",
          passValue = "WHISPER",
        }
      }
    },
    quickcheck = {
      name = L["Quick Check"],
      desc = L["A quick report that shows who does not have flasks, elixirs or food."],
      type = 'group',
      args = {
        self = {
          name = L["Self"],
          desc = L["Reports result only to yourself."],
          type = 'execute',
          func = "QuickCheck",
          passValue = "SELF",
        },
        party = {
          name = L["Party"],
          desc = L["Reports result to your party."],
          type = 'execute',
          func = "QuickCheck",
          disabled = function() return GetNumPartyMembers()==0 end,
          passValue = "PARTY",
        },
        raid = {
          name = L["Raid"],
          desc = L["Reports result to your raid."],
          type = 'execute',
          func = "QuickCheck",
          disabled = function() return GetNumRaidMembers()==0 end,
          passValue = "RAID",
        },
        guild = {
          name = L["Guild"],
          desc = L["Reports result to guild chat."],
          type = 'execute',
          func = "QuickCheck",
          passValue = "GUILD",
        },
        officer = {
          name = L["Officer"],
          desc = L["Reports result to officer chat."],
          type = 'execute',
          func = "QuickCheck",
          passValue = "OFFICER",
        },
        whisper = {
          name = L["Whisper"],
          desc = L["Reports result to the currently targeted individual."],
          type = 'execute',
          func = "QuickCheck",
          passValue = "WHISPER",
        }
      }
    },
    settings = {
      name = L["Settings"],
      desc = L["Mod Settings"],
      type = 'group',
      args = {
        polymorph = {
          name  = BS["Polymorph"],
          desc = L["Reports if hostile polymorphs or shackles are broken and by who."],
          type = 'toggle',
          get = function() return addon.db.profile.PolyBreak end,
          set = function(v)
            addon.db.profile.PolyBreak=v 
            if v then
              parser:RegisterInfoType("BigBrother","dispel", addon.CCBreaker)
            else
              parser:UnregisterInfoType("BigBrother","dispel")
            end
          end,
          map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
        },
        misdirect = {
          name  = L["Misdirect"],
          desc = L["Reports who gains misdirection."],
          type = 'toggle',
          get = function() return addon.db.profile.Misdirect end,
          set = function(v) addon.db.profile.Misdirect = v end,
          map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
        },
        polyoutput = {
          name = L["Polymorph/Misdirect Output"],
          desc = L["Set where the polymorph/misdirect output is sent"],
          type = 'group',
          args = {
            self = {
              name = L["Self"],
              desc = L["Reports result only to yourself."],
              type = 'toggle',
              get = function() return addon.db.profile.PolyOut[1] end,
              set = function(v) addon.db.profile.PolyOut[1] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            party = {
              name = L["Party"],
              desc = L["Reports result to your party."],
              type = 'toggle',
              get = function() return addon.db.profile.PolyOut[2] end,
              set = function(v) addon.db.profile.PolyOut[2] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            raid = {
              name = L["Raid"],
              desc = L["Reports result to your raid."],
              type = 'toggle',
              get = function() return addon.db.profile.PolyOut[3] end,
              set = function(v) addon.db.profile.PolyOut[3] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            guild = {
              name = L["Guild"],
              desc = L["Reports result to guild chat."],
              type = 'toggle',
              get = function() return addon.db.profile.PolyOut[4] end,
              set = function(v) addon.db.profile.PolyOut[4] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            officer = {
              name = L["Officer"],
              desc = L["Reports result to officer chat."],
              type = 'toggle',
              get = function() return addon.db.profile.PolyOut[5] end,
              set = function(v) addon.db.profile.PolyOut[5] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
          }
        },
        checks = {
          name = L["Checks"],
          desc = L["Set whether Flasks, Elixirs and Food are included in flaskcheck/quickcheck"],
          type = 'group',
          args = {
            group1 = {
              name  = L["Flasks"],
              desc = L["Flasks"],
              type = 'toggle',
              get = function() return addon.db.profile.CheckFlasks end,
              set = function(v) addon.db.profile.CheckFlasks = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group2 = {
              name  = L["Elixirs"],
              desc = L["Elixirs"],
              type = 'toggle',
              get = function() return addon.db.profile.CheckElixirs end,
              set = function(v) addon.db.profile.CheckElixirs = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group3 = {
              name  = L["Food Buffs"],
              desc = L["Food Buffs"],
              type = 'toggle',
              get = function() return addon.db.profile.CheckFood end,
              set = function(v) addon.db.profile.CheckFood = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
          }
        },
        groups = {
          name = L["Raid Groups"],
          desc = L["Set which raid groups are checked for buffs"],
          type = 'group',
          args = {
            group1 = {
              name  = L["Group 1"],
              desc = L["Group 1"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[1] end,
              set = function(v) addon.db.profile.Groups[1] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group2 = {
              name  = L["Group 2"],
              desc = L["Group 2"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[2] end,
              set = function(v) addon.db.profile.Groups[2] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group3 = {
              name  = L["Group 3"],
              desc = L["Group 3"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[3] end,
              set = function(v) addon.db.profile.Groups[3] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group4 = {
              name  = L["Group 4"],
              desc = L["Group 4"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[4] end,
              set = function(v) addon.db.profile.Groups[4] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group5 = {
              name  = L["Group 5"],
              desc = L["Group 5"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[5] end,
              set = function(v) addon.db.profile.Groups[5] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group6 = {
              name  = L["Group 6"],
              desc = L["Group 6"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[6] end,
              set = function(v) addon.db.profile.Groups[6] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group7 = {
              name  = L["Group 7"],
              desc = L["Group 7"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[7] end,
              set = function(v) addon.db.profile.Groups[7] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
            group8 = {
              name  = L["Group 8"],
              desc = L["Group 8"],
              type = 'toggle',
              get = function() return addon.db.profile.Groups[8] end,
              set = function(v) addon.db.profile.Groups[8] = v end,
              map = { [false] = "|cffff4040Disabled|r", [true] = "|cff40ff40Enabled|r" }
            },
          }
        },
      }
    },
    buffcheck = {
      name = L["BuffCheck"],
      desc = L["Pops up a window to check various raid/elixir buffs (drag the bottom to resize)."],
      type = 'execute',
      func = function() ToggleBuffWindow() end,
    }
  }
}

addon.OnMenuRequest = options


function addon:OnInitialize()
  self:RegisterChatCommand("/bb", "/bigbrother", options, "BIGBROTHER")
end

function addon:OnEnable()
  parser:RegisterInfoType("BigBrother","dispel", addon.CCBreaker)
  parser:RegisterInfoType("BigBrother_BuffWindow","dispel", addon.BuffUpdating)
  self:RegisterEvent("SpecialEvents_UnitBuffGained")
  self:RegisterEvent("SpecialEvents_UnitBuffLost", "BuffUpdating")
  self:OnProfileEnable()
end

function addon:OnProfileDisable()
end

function addon:OnProfileEnable()
end

function addon:SpecialEvents_UnitBuffGained(unit, name, index, count, icon, rank)
  if name == BS["Misdirection"] then
    if self.db.profile.Misdirect then
      self:SendMessageMisdirect(UnitName(unit))
    end
  else
    self:BuffUpdating()
  end
end

function addon:SendMessageList(Pre,List,Where)
  if #List > 0 then
    if Where == "SELF" then
      self:Print(string.format(color, Pre..":") .. " " .. table.concat(List, ", "))
    elseif Where == "WHISPER" then
      local theTarget = UnitName("playertarget")
      if theTarget == nil then
         theTarget = UnitName("player")
      end
      SendChatMessage(Pre..": "..table.concat(List, ", "),Where,nil,theTarget)
    else
      SendChatMessage(Pre..": "..table.concat(List, ", "),Where)
    end
  end
end

function addon:SendMessageMisdirect(Unitname)
  for i, v in ipairs(addon.db.profile.PolyOut) do
    if v then
      if i==1 then
        -- self
        BigBrother:Print(L["Misdirection on |cff40ff40%s|r."]:format(Unitname));
      elseif i==2 then
        -- party
        SendChatMessage(L["Misdirection on %s"]:format(Unitname),"PARTY")
      elseif i==3 then
        -- raid
        SendChatMessage(L["Misdirection on %s"]:format(Unitname),"RAID")
      elseif i==4 then
        -- guild
        SendChatMessage(L["Misdirection on %s"]:format(Unitname),"GUILD")
      elseif i==5 then
        -- officer
        SendChatMessage(L["Misdirection on %s"]:format(Unitname),"OFFICER")
      end
    end
  end
end


function addon:SendMessageCC(CCType, Victim, Source, Skill, VictimHostile, SourceHostile)
  -- build message
  local message = ""
  if Source and Skill then
    message = string.format(L["%s on %s removed by %s's %s"], CCType, Victim, Source, Skill)
  elseif Source then
    message = string.format(L["%s on %s removed by %s"], CCType, Victim, Source)
  else
    message = string.format(L["%s on %s is removed"], CCType, Victim)
  end
  
  for i, v in ipairs(addon.db.profile.PolyOut) do
    if v then
      if i==1 then
        -- self
        if Source and Skill then
          addon:Print(L["%s on %s|r removed by %s|r's %s"]:format(CCType, VictimHostile..Victim, SourceHostile..Source, Skill))
        elseif Source then
          addon:Print(L["%s on %s|r removed by %s|r"]:format(CCType, VictimHostile..Victim, SourceHostile..Source))
        else
          addon:Print(L["%s on %s|r is removed"]:format(CCType, VictimHostile..Victim))
        end
      elseif i==2 then
        -- party
        SendChatMessage(message,"PARTY")
      elseif i==3 then
        -- raid
        SendChatMessage(message,"RAID")
      elseif i==4 then
        -- guild
        SendChatMessage(message,"GUILD")
      elseif i==5 then
        -- officer
        SendChatMessage(message,"OFFICER")
      end
    end
  end
end


function addon:HasBuff(player,MissingBuffList)
  for k, v in pairs(MissingBuffList) do
    if v==player then
      table.remove(MissingBuffList,k)
    end
  end
end

function addon:FlaskCheck(Where)
  self:ConsumableCheck(Where, true)
end

function addon:QuickCheck(Where)
  self:ConsumableCheck(Where, false)
end


function addon:ConsumableCheck(Where,Full)
  local totalFlaskCount = 0
  local totalElixirCount = 0
  local numElixirs = 0
  local MissingFlaskList={}
  local MissingElixirList={}
  local MissingFoodList={}

  for unit in RL:IterateRoster(false) do
    if addon.db.profile.Groups[unit.subgroup] then
      table.insert(MissingFlaskList,unit.name)
      table.insert(MissingFoodList,unit.name)
    end
  end

  --print the flask list and determine who has no flask
  if addon.db.profile.CheckFlasks then
    for i, v in ipairs(buffs) do
      local t = self:BuffPlayerList(v,MissingFlaskList)
      if Full then
        self:SendMessageList(v, t, Where)
      end
      totalFlaskCount = totalFlaskCount + #t
    end
    
    if Full then
      self:SendMessageList(L["No Flask"], MissingFlaskList, Where)
    end
  end
  
  --use this to print out who has what elixir, and who has no elixirs
  if addon.db.profile.CheckElixirs then
    for i, v in ipairs(elixirs) do
      local t = self:BuffPlayerList(v, MissingFlaskList)
      if Full then
        self:SendMessageList(v, t, Where)
      end
      totalElixirCount = totalElixirCount + #t
    end  
    
    --now figure out who has only one elixir
    for unit in RL:IterateRoster(false) do
      if addon.db.profile.Groups[unit.subgroup] then
        numElixirs = 0
        for i, v in ipairs(elixirs) do
            if aura:UnitHasBuff(unit.unitid, v) then
              numElixirs = numElixirs + 1
            end
        end
        if numElixirs == 1 then
            table.insert(MissingElixirList,unit.name)
        end
      end
    end

    self:SendMessageList(L["Only One Elixir"], MissingElixirList, Where)
    self:SendMessageList(L["No Flask or Elixir"], MissingFlaskList, Where)
  end
  
  --check for missing food
  if addon.db.profile.CheckFood then
    for i, v in ipairs(elixirs) do
      local t = self:BuffPlayerList(L["Well Fed"], MissingFoodList)
      local t = self:BuffPlayerList(L["\"Well Fed\""], MissingFoodList)
	  local t = self:BuffPlayerList(L["Enlightened"], MissingFoodList)
	  local t = self:BuffPlayerList(L["Electrified"], MissingFoodList)
    end
    self:SendMessageList(L["No Food Buff"], MissingFoodList, Where)
  end  
end

function addon:BuffPlayerList(buffname,MissingBuffList)
  local list = {}
  for unit in RL:IterateRoster(false) do
    if aura:UnitHasBuff(unit.unitid, buffname) then
      table.insert(list, unit.name)
      self:HasBuff(unit.name,MissingBuffList)
    end
  end
  return list
end

function addon:CCBreaker(info)
  --Since we don't register the event for when variables are loaded
  if not addon.db.profile.PolyBreak then
    parser:UnregisterInfoType("BigBrother","dispel")
    return
  end
  
  -- Polymorph
  local Polymorph=string.find(info.skill,BS["Polymorph"])
  if Polymorph and info.source then
    local SourceHostile,VictimHostile

    if info.source==ParserLib_SELF then
      info.source=UnitName("PLAYER")
    end
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end

    if RL:GetUnitIDFromName(info.victim) then
      --We don't care about friendlies being polymorphed
      return
    else
      VictimHostile="|cffff4040"
    end

    if RL:GetUnitIDFromName(info.source) then
      SourceHostile="|cff40ff40"
    else
      SourceHostile="|cffff4040"
    end
    
    addon:SendMessageCC(L["Polymorph"], info.victim, info.source, info.sourceSkill, VictimHostile, SourceHostile)
    
  elseif Polymorph then
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end

    if RL:GetUnitIDFromName(info.victim) then
      --We don't care about friendlies being polymorphed
      return
    else
      VictimHostile="|cffff4040"
    end
    
    addon:SendMessageCC(L["Polymorph"], info.victim, nil, nil, VictimHostile, nil)
  end
  
  -- Shackle
  local Shackle=string.find(info.skill,BS["Shackle Undead"])
  if Shackle and info.source then
    local SourceHostile,VictimHostile

    if info.source==ParserLib_SELF then
      info.source=UnitName("PLAYER")
    end
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end

    VictimHostile="|cffff4040"
    
    if RL:GetUnitIDFromName(info.source) then
      SourceHostile="|cff40ff40"
    else
      SourceHostile="|cffff4040"
    end

    addon:SendMessageCC(L["Shackle"], info.victim, info.source, info.sourceSkill, VictimHostile, SourceHostile)
  elseif Shackle then
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end
    
    VictimHostile="|cffff4040"
    
    addon:SendMessageCC(L["Shackle"], info.victim, nil, nil, VictimHostile, nil)
  end
  
  -- Hibernate
  local Shackle=string.find(info.skill,BS["Hibernate"])
  if Shackle and info.source then
    local SourceHostile,VictimHostile

    if info.source==ParserLib_SELF then
      info.source=UnitName("PLAYER")
    end
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end

    VictimHostile="|cffff4040"
    
    if RL:GetUnitIDFromName(info.source) then
      SourceHostile="|cff40ff40"
    else
      SourceHostile="|cffff4040"
    end

    addon:SendMessageCC(L["Hibernate"], info.victim, info.source, info.sourceSkill, VictimHostile, SourceHostile)
  elseif Shackle then
    if info.victim==ParserLib_SELF then
      info.victim=UnitName("PLAYER")
    end
    
    VictimHostile="|cffff4040"
    
    addon:SendMessageCC(L["Hibernate"], info.victim, nil, nil, VictimHostile, nil)
  end
  
  
end


function addon:BuffUpdating(info)
  if BigBrother_BuffWindow and BigBrother_BuffWindow:IsShown() then
    addon:ScheduleEvent(BuffWindow_Update,0.25)
  end
end

